@@ -12,6 +12,7 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include <mntent.h>
 
 #include "attach_options.h"
 
@@ -28,12 +29,6 @@
 #include "syscall_wrappers.h"
 #include "utils.h"
 
-#if IS_BIONIC
-#include "lxcmntent.h"
-#else
-#include <mntent.h>
-#endif
-
 #if !HAVE_STRLCPY
 #include "strlcpy.h"
 #endif
@@ -45,6 +40,7 @@
 
 #define CRIU_IN_FLIGHT_SUPPORT	"2.4"
 #define CRIU_EXTERNAL_NOT_VETH	"2.8"
+#define CRIU_EXTERNAL_NETDEV	"3.15"
 
 lxc_log_define(criu, lxc);
 
@@ -320,7 +316,7 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 				return log_error_errno(-ENOMEM, ENOMEM, "Failed to duplicate limit cgroup path");
 		}
 
-		tmp = path_simplify(cgroup_base_path);
+		tmp = lxc_path_simplify(cgroup_base_path);
 		if (!tmp)
 			return log_error_errno(-ENOMEM, ENOMEM, "Failed to remove extraneous slashes from \"%s\"", cgroup_base_path);
 		free_move_ptr(cgroup_base_path, tmp);
@@ -550,8 +546,22 @@ static int exec_criu(struct cgroup_ops *cgroup_ops, struct lxc_conf *conf,
 
 				TRACE("Added macvlan device entry %s", buf);
 
+				break;
+			case LXC_NET_PHYS:
+				if (cmp_version(opts->criu_version, CRIU_EXTERNAL_NETDEV) < 0)
+					return syserror_set(-EOPNOTSUPP, "Restoring physical network devices not supported");
+
+				if (is_empty_string(netdev->link))
+					return syserror_set(-EINVAL, "Specifying link is required");
+
+				ret = strnprintf(buf, sizeof(buf), "netdev[%s]:%s", eth, netdev->link);
+				if (ret < 0)
+					return syserror_set(-EIO, "Failed to append phys device name");
+
+				TRACE("Added phys device entry %s", buf);
 				break;
 			case LXC_NET_NONE:
+				__fallthrough;
 			case LXC_NET_EMPTY:
 				break;
 			default:
@@ -825,6 +835,7 @@ static bool criu_ok(struct lxc_container *c, char **criu_version)
 		case LXC_NET_VETH:
 		case LXC_NET_NONE:
 		case LXC_NET_EMPTY:
+		case LXC_NET_PHYS:
 		case LXC_NET_MACVLAN:
 			break;
 		default:
@@ -1208,7 +1219,7 @@ static bool do_dump(struct lxc_container *c, char *mode, struct migrate_opts *op
 		return false;
 	}
 
-	if (mkdir_p(opts->directory, 0700) < 0)
+	if (lxc_mkdir_p(opts->directory, 0700) < 0)
 		goto fail;
 
 	pid = fork();
