@@ -315,6 +315,7 @@ enum lxc_hostarch_t {
 	lxc_seccomp_arch_mipsel64n32,
 	lxc_seccomp_arch_s390x,
 	lxc_seccomp_arch_s390,
+	lxc_seccomp_arch_loongarch64,
 	lxc_seccomp_arch_unknown = 999,
 };
 
@@ -349,6 +350,8 @@ static int get_hostarch(void)
 		return lxc_seccomp_arch_s390x;
 	else if (strnequal(uts.machine, "s390", 4))
 		return lxc_seccomp_arch_s390;
+	else if (strnequal(uts.machine, "loongarch64", 11))
+		return lxc_seccomp_arch_loongarch64;
 	return lxc_seccomp_arch_unknown;
 }
 
@@ -421,6 +424,11 @@ static scmp_filter_ctx get_new_ctx(enum lxc_hostarch_t n_arch, uint32_t default_
 	case lxc_seccomp_arch_s390:
 		arch = SCMP_ARCH_S390;
 		break;
+#endif
+#ifdef SCMP_ARCH_LOONGARCH64
+	case lxc_seccomp_arch_loongarch64:
+		arch = SCMP_ARCH_LOONGARCH64;
+		break;
 #endif
 	default:
 		return NULL;
@@ -967,6 +975,17 @@ static int parse_config_v2(FILE *f, char *line, size_t *line_bufsz, struct lxc_c
 
 				cur_rule_arch = lxc_seccomp_arch_s390;
 			}
+#endif
+#ifdef SCMP_ARCH_LOONGARCH64
+                        else if (strequal(line, "[loongarch64]") ||
+                                 strequal(line, "[LOONGARCH64]")) {
+                                if (native_arch != lxc_seccomp_arch_loongarch64) {
+                                        cur_rule_arch = lxc_seccomp_arch_unknown;
+                                        continue;
+                                }
+
+                                cur_rule_arch = lxc_seccomp_arch_loongarch64;
+                        }
 #endif
 			else {
 				goto bad_arch;
@@ -1282,8 +1301,8 @@ int lxc_seccomp_load(struct lxc_conf *conf)
 			return -1;
 		}
 
-		if (fd_make_nonblocking(ret))
-			return log_error_errno(-1, errno, "Failed to make seccomp listener fd non-blocking");;
+		if (fd_make_blocking(ret))
+			return log_error_errno(-1, errno, "Failed to make seccomp listener fd blocking");
 
 		conf->seccomp.notifier.notify_fd = ret;
 		TRACE("Retrieved new seccomp listener fd %d", ret);
@@ -1373,6 +1392,7 @@ int seccomp_notify_cleanup_handler(int fd, void *data)
 	 * seccomp notify handler through the command socket (e.g. for attach)
 	 * and so we won't touch the container's config.
 	 */
+	close(fd);
 #endif
 	return 0;
 }
